# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "ExperimentArmErrorEnum",
    },
)


class ExperimentArmErrorEnum(proto.Message):
    r"""Container for enum describing possible experiment arm error."""

    class ExperimentArmError(proto.Enum):
        r"""Enum describing possible experiment arm errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED (2):
                Number of experiment arms is above limit.
            INVALID_CAMPAIGN_STATUS (3):
                Cannot add campaign with invalid status to
                the experiment arm.
            DUPLICATE_EXPERIMENT_ARM_NAME (4):
                Cannot add duplicate experiment arm name in
                one experiment.
            CANNOT_SET_TREATMENT_ARM_CAMPAIGN (5):
                Cannot set campaigns of treatment experiment
                arm.
            CANNOT_MODIFY_CAMPAIGN_IDS (6):
                Cannot edit campaign ids in trial arms in non
                SETUP experiment.
            CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET (7):
                Cannot modify the campaigns in the control
                arm if there is not a suffix set in the trial.
            CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START (8):
                Traffic split related settings (like traffic
                share bounds) can't be modified after the trial
                has started.
            CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET (9):
                Cannot use shared budget on experiment's
                control campaign.
            CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET (10):
                Cannot use custom budget on experiment's
                control campaigns.
            CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED (11):
                Cannot have enable_dynamic_assets turned on in experiment's
                campaigns.
            UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE (12):
                Cannot use campaign's advertising channel sub
                type in experiment.
            CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE (13):
                Experiment date range must be within base
                campaign's date range.
            BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS (14):
                Bidding strategy is not supported in
                experiments.
            TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE (15):
                Traffic split is not supported for some
                channel types.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXPERIMENT_ARM_COUNT_LIMIT_EXCEEDED = 2
        INVALID_CAMPAIGN_STATUS = 3
        DUPLICATE_EXPERIMENT_ARM_NAME = 4
        CANNOT_SET_TREATMENT_ARM_CAMPAIGN = 5
        CANNOT_MODIFY_CAMPAIGN_IDS = 6
        CANNOT_MODIFY_CAMPAIGN_WITHOUT_SUFFIX_SET = 7
        CANNOT_MUTATE_TRAFFIC_SPLIT_AFTER_START = 8
        CANNOT_ADD_CAMPAIGN_WITH_SHARED_BUDGET = 9
        CANNOT_ADD_CAMPAIGN_WITH_CUSTOM_BUDGET = 10
        CANNOT_ADD_CAMPAIGNS_WITH_DYNAMIC_ASSETS_ENABLED = 11
        UNSUPPORTED_CAMPAIGN_ADVERTISING_CHANNEL_SUB_TYPE = 12
        CANNOT_ADD_BASE_CAMPAIGN_WITH_DATE_RANGE = 13
        BIDDING_STRATEGY_NOT_SUPPORTED_IN_EXPERIMENTS = 14
        TRAFFIC_SPLIT_NOT_SUPPORTED_FOR_CHANNEL_TYPE = 15


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "QueryErrorEnum",
    },
)


class QueryErrorEnum(proto.Message):
    r"""Container for enum describing possible query errors."""

    class QueryError(proto.Enum):
        r"""Enum describing possible query errors.

        Values:
            UNSPECIFIED (0):
                Name unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            QUERY_ERROR (50):
                Returned if all other query error reasons are
                not applicable.
            BAD_ENUM_CONSTANT (18):
                A condition used in the query references an
                invalid enum constant.
            BAD_ESCAPE_SEQUENCE (7):
                Query contains an invalid escape sequence.
            BAD_FIELD_NAME (12):
                Field name is invalid.
            BAD_LIMIT_VALUE (15):
                Limit value is invalid (for example, not a
                number)
            BAD_NUMBER (5):
                Encountered number can not be parsed.
            BAD_OPERATOR (3):
                Invalid operator encountered.
            BAD_PARAMETER_NAME (61):
                Parameter unknown or not supported.
            BAD_PARAMETER_VALUE (62):
                Parameter have invalid value.
            BAD_RESOURCE_TYPE_IN_FROM_CLAUSE (45):
                Invalid resource type was specified in the
                FROM clause.
            BAD_SYMBOL (2):
                Non-ASCII symbol encountered outside of
                strings.
            BAD_VALUE (4):
                Value is invalid.
            DATE_RANGE_TOO_WIDE (36):
                Date filters fail to restrict date to a range
                smaller than 31 days. Applicable if the query is
                segmented by date.
            DATE_RANGE_TOO_NARROW (60):
                Filters on date/week/month/quarter have a
                start date after end date.
            EXPECTED_AND (30):
                Expected AND between values with BETWEEN
                operator.
            EXPECTED_BY (14):
                Expecting ORDER BY to have BY.
            EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE (37):
                There was no dimension field selected.
            EXPECTED_FILTERS_ON_DATE_RANGE (55):
                Missing filters on date related fields.
            EXPECTED_FROM (44):
                Missing FROM clause.
            EXPECTED_LIST (41):
                The operator used in the conditions requires
                the value to be a list.
            EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE (16):
                Fields used in WHERE or ORDER BY clauses are
                missing from the SELECT clause.
            EXPECTED_SELECT (13):
                SELECT is missing at the beginning of query.
            EXPECTED_SINGLE_VALUE (42):
                A list was passed as a value to a condition
                whose operator expects a single value.
            EXPECTED_VALUE_WITH_BETWEEN_OPERATOR (29):
                Missing one or both values with BETWEEN
                operator.
            INVALID_DATE_FORMAT (38):
                Invalid date format. Expected 'YYYY-MM-DD'.
            MISALIGNED_DATE_FOR_FILTER (64):
                Misaligned date value for the filter. The
                date should be the start of a week/month/quarter
                if the filtered field is
                segments.week/segments.month/segments.quarter.
            INVALID_STRING_VALUE (57):
                Value passed was not a string when it should
                have been. For example, it was a number or
                unquoted literal.
            INVALID_VALUE_WITH_BETWEEN_OPERATOR (26):
                A String value passed to the BETWEEN operator
                does not parse as a date.
            INVALID_VALUE_WITH_DURING_OPERATOR (22):
                The value passed to the DURING operator is
                not a Date range literal
            INVALID_VALUE_WITH_LIKE_OPERATOR (56):
                An invalid value was passed to the LIKE
                operator.
            OPERATOR_FIELD_MISMATCH (35):
                An operator was provided that is inapplicable
                to the field being filtered.
            PROHIBITED_EMPTY_LIST_IN_CONDITION (28):
                A Condition was found with an empty list.
            PROHIBITED_ENUM_CONSTANT (54):
                A condition used in the query references an
                unsupported enum constant.
            PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE (31):
                Fields that are not allowed to be selected
                together were included in the SELECT clause.
            PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE (40):
                A field that is not orderable was included in
                the ORDER BY clause.
            PROHIBITED_FIELD_IN_SELECT_CLAUSE (23):
                A field that is not selectable was included
                in the SELECT clause.
            PROHIBITED_FIELD_IN_WHERE_CLAUSE (24):
                A field that is not filterable was included
                in the WHERE clause.
            PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE (43):
                Resource type specified in the FROM clause is
                not supported by this service.
            PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE (48):
                A field that comes from an incompatible
                resource was included in the SELECT clause.
            PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE (58):
                A field that comes from an incompatible
                resource was included in the WHERE clause.
            PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE (49):
                A metric incompatible with the main resource
                or other selected segmenting resources was
                included in the SELECT or WHERE clause.
            PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE (51):
                A segment incompatible with the main resource
                or other selected segmenting resources was
                included in the SELECT or WHERE clause.
            PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE (53):
                A segment in the SELECT clause is
                incompatible with a metric in the SELECT or
                WHERE clause.
            LIMIT_VALUE_TOO_LOW (25):
                The value passed to the limit clause is too
                low.
            PROHIBITED_NEWLINE_IN_STRING (8):
                Query has a string containing a newline
                character.
            PROHIBITED_VALUE_COMBINATION_IN_LIST (10):
                List contains values of different types.
            PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR (21):
                The values passed to the BETWEEN operator are
                not of the same type.
            STRING_NOT_TERMINATED (6):
                Query contains unterminated string.
            TOO_MANY_SEGMENTS (34):
                Too many segments are specified in SELECT
                clause.
            UNEXPECTED_END_OF_QUERY (9):
                Query is incomplete and cannot be parsed.
            UNEXPECTED_FROM_CLAUSE (47):
                FROM clause cannot be specified in this
                query.
            UNRECOGNIZED_FIELD (32):
                Query contains one or more unrecognized
                fields.
            UNEXPECTED_INPUT (11):
                Query has an unexpected extra part.
            REQUESTED_METRICS_FOR_MANAGER (59):
                Metrics cannot be requested for a manager
                account. To retrieve metrics, issue separate
                requests against each client account under the
                manager account.
            FILTER_HAS_TOO_MANY_VALUES (63):
                The number of values (right-hand-side
                operands) in a filter exceeds the limit.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        QUERY_ERROR = 50
        BAD_ENUM_CONSTANT = 18
        BAD_ESCAPE_SEQUENCE = 7
        BAD_FIELD_NAME = 12
        BAD_LIMIT_VALUE = 15
        BAD_NUMBER = 5
        BAD_OPERATOR = 3
        BAD_PARAMETER_NAME = 61
        BAD_PARAMETER_VALUE = 62
        BAD_RESOURCE_TYPE_IN_FROM_CLAUSE = 45
        BAD_SYMBOL = 2
        BAD_VALUE = 4
        DATE_RANGE_TOO_WIDE = 36
        DATE_RANGE_TOO_NARROW = 60
        EXPECTED_AND = 30
        EXPECTED_BY = 14
        EXPECTED_DIMENSION_FIELD_IN_SELECT_CLAUSE = 37
        EXPECTED_FILTERS_ON_DATE_RANGE = 55
        EXPECTED_FROM = 44
        EXPECTED_LIST = 41
        EXPECTED_REFERENCED_FIELD_IN_SELECT_CLAUSE = 16
        EXPECTED_SELECT = 13
        EXPECTED_SINGLE_VALUE = 42
        EXPECTED_VALUE_WITH_BETWEEN_OPERATOR = 29
        INVALID_DATE_FORMAT = 38
        MISALIGNED_DATE_FOR_FILTER = 64
        INVALID_STRING_VALUE = 57
        INVALID_VALUE_WITH_BETWEEN_OPERATOR = 26
        INVALID_VALUE_WITH_DURING_OPERATOR = 22
        INVALID_VALUE_WITH_LIKE_OPERATOR = 56
        OPERATOR_FIELD_MISMATCH = 35
        PROHIBITED_EMPTY_LIST_IN_CONDITION = 28
        PROHIBITED_ENUM_CONSTANT = 54
        PROHIBITED_FIELD_COMBINATION_IN_SELECT_CLAUSE = 31
        PROHIBITED_FIELD_IN_ORDER_BY_CLAUSE = 40
        PROHIBITED_FIELD_IN_SELECT_CLAUSE = 23
        PROHIBITED_FIELD_IN_WHERE_CLAUSE = 24
        PROHIBITED_RESOURCE_TYPE_IN_FROM_CLAUSE = 43
        PROHIBITED_RESOURCE_TYPE_IN_SELECT_CLAUSE = 48
        PROHIBITED_RESOURCE_TYPE_IN_WHERE_CLAUSE = 58
        PROHIBITED_METRIC_IN_SELECT_OR_WHERE_CLAUSE = 49
        PROHIBITED_SEGMENT_IN_SELECT_OR_WHERE_CLAUSE = 51
        PROHIBITED_SEGMENT_WITH_METRIC_IN_SELECT_OR_WHERE_CLAUSE = 53
        LIMIT_VALUE_TOO_LOW = 25
        PROHIBITED_NEWLINE_IN_STRING = 8
        PROHIBITED_VALUE_COMBINATION_IN_LIST = 10
        PROHIBITED_VALUE_COMBINATION_WITH_BETWEEN_OPERATOR = 21
        STRING_NOT_TERMINATED = 6
        TOO_MANY_SEGMENTS = 34
        UNEXPECTED_END_OF_QUERY = 9
        UNEXPECTED_FROM_CLAUSE = 47
        UNRECOGNIZED_FIELD = 32
        UNEXPECTED_INPUT = 11
        REQUESTED_METRICS_FOR_MANAGER = 59
        FILTER_HAS_TOO_MANY_VALUES = 63


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates how to create a new customer under a given
manager account.

Note: this example must be run using the credentials of a Google Ads manager
account. By default, the new account will only be accessible via the manager
account.
"""


import argparse
import sys
from datetime import datetime

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.customer import Customer
from google.ads.googleads.v22.services.services.customer_service.client import (
    CustomerServiceClient,
)
from google.ads.googleads.v22.services.types.customer_service import (
    CreateCustomerClientResponse,
)


# [START create_customer]
def main(client: GoogleAdsClient, manager_customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        manager_customer_id: a manager client customer ID.
    """
    customer_service: CustomerServiceClient = client.get_service(
        "CustomerService"
    )
    customer: Customer = client.get_type("Customer")
    now: str = datetime.today().strftime("%Y%m%d %H:%M:%S")
    customer.descriptive_name = f"Account created with CustomerService on {now}"
    # For a list of valid currency codes and time zones see this documentation:
    # https://developers.google.com/google-ads/api/reference/data/codes-formats
    customer.currency_code = "USD"
    customer.time_zone = "America/New_York"
    # The below values are optional. For more information about URL
    # options see: https://support.google.com/google-ads/answer/6305348
    customer.tracking_url_template = "{lpurl}?device={device}"
    customer.final_url_suffix = (
        "keyword={keyword}&matchtype={matchtype}&adgroupid={adgroupid}"
    )

    response: CreateCustomerClientResponse = (
        customer_service.create_customer_client(
            customer_id=manager_customer_id, customer_client=customer
        )
    )
    print(
        f'Customer created with resource name "{response.resource_name}" '
        f'under manager account with ID "{manager_customer_id}".'
    )
    # [END create_customer]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=("Creates a new client under the given manager.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-m",
        "--manager_customer_id",
        type=str,
        required=True,
        help="A Google Ads customer ID for the "
        "manager account under which the new customer will "
        "be created.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.manager_customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import asset_usage


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupTopCombinationView",
        "AssetGroupAssetCombinationData",
    },
)


class AssetGroupTopCombinationView(proto.Message):
    r"""A view on the usage of asset group asset top combinations.

    Attributes:
        resource_name (str):
            Output only. The resource name of the asset group top
            combination view. AssetGroup Top Combination view resource
            names have the form:
            \`"customers/{customer_id}/assetGroupTopCombinationViews/{asset_group_id}~{asset_combination_category}".
        asset_group_top_combinations (MutableSequence[google.ads.googleads.v22.resources.types.AssetGroupAssetCombinationData]):
            Output only. The top combinations of assets
            that served together.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_group_top_combinations: MutableSequence[
        "AssetGroupAssetCombinationData"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="AssetGroupAssetCombinationData",
    )


class AssetGroupAssetCombinationData(proto.Message):
    r"""Asset group asset combination data

    Attributes:
        asset_combination_served_assets (MutableSequence[google.ads.googleads.v22.common.types.AssetUsage]):
            Output only. Served assets.
    """

    asset_combination_served_assets: MutableSequence[asset_usage.AssetUsage] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=asset_usage.AssetUsage,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignDraftStatusEnum",
    },
)


class CampaignDraftStatusEnum(proto.Message):
    r"""Container for enum describing possible statuses of a campaign
    draft.

    """

    class CampaignDraftStatus(proto.Enum):
        r"""Possible statuses of a campaign draft.

        Values:
            UNSPECIFIED (0):
                The status has not been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PROPOSED (2):
                Initial state of the draft, the advertiser
                can start adding changes with no effect on
                serving.
            REMOVED (3):
                The campaign draft is removed.
            PROMOTING (5):
                Advertiser requested to promote draft's
                changes back into the original campaign.
                Advertiser can poll the long running operation
                returned by the promote action to see the status
                of the promotion.
            PROMOTED (4):
                The process to merge changes in the draft
                back to the original campaign has completed
                successfully.
            PROMOTE_FAILED (6):
                The promotion failed after it was partially
                applied. Promote cannot be attempted again
                safely, so the issue must be corrected in the
                original campaign.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PROPOSED = 2
        REMOVED = 3
        PROMOTING = 5
        PROMOTED = 4
        PROMOTE_FAILED = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ChangeStatusResourceTypeEnum",
    },
)


class ChangeStatusResourceTypeEnum(proto.Message):
    r"""Container for enum describing supported resource types for
    the ChangeStatus resource.

    """

    class ChangeStatusResourceType(proto.Enum):
        r"""Enum listing the resource types support by the ChangeStatus
        resource.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents an
                unclassified resource unknown in this version.
            AD_GROUP (3):
                An AdGroup resource change.
            AD_GROUP_AD (4):
                An AdGroupAd resource change.
            AD_GROUP_CRITERION (5):
                An AdGroupCriterion resource change.
            CAMPAIGN (6):
                A Campaign resource change.
            CAMPAIGN_CRITERION (7):
                A CampaignCriterion resource change.
            CAMPAIGN_BUDGET (8):
                A CampaignBudget resource change.
            FEED (9):
                A Feed resource change.
            FEED_ITEM (10):
                A FeedItem resource change.
            AD_GROUP_FEED (11):
                An AdGroupFeed resource change.
            CAMPAIGN_FEED (12):
                A CampaignFeed resource change.
            AD_GROUP_BID_MODIFIER (13):
                An AdGroupBidModifier resource change.
            SHARED_SET (14):
                A SharedSet resource change.
            CAMPAIGN_SHARED_SET (15):
                A CampaignSharedSet resource change.
            ASSET (16):
                An Asset resource change.
            CUSTOMER_ASSET (17):
                A CustomerAsset resource change.
            CAMPAIGN_ASSET (18):
                A CampaignAsset resource change.
            AD_GROUP_ASSET (19):
                An AdGroupAsset resource change.
            COMBINED_AUDIENCE (20):
                A CombinedAudience resource change.
            ASSET_GROUP (21):
                An AssetGroup resource change.
            ASSET_SET (22):
                An AssetSet resource change.
            CAMPAIGN_ASSET_SET (23):
                A CampaignAssetSet resource change.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_GROUP = 3
        AD_GROUP_AD = 4
        AD_GROUP_CRITERION = 5
        CAMPAIGN = 6
        CAMPAIGN_CRITERION = 7
        CAMPAIGN_BUDGET = 8
        FEED = 9
        FEED_ITEM = 10
        AD_GROUP_FEED = 11
        CAMPAIGN_FEED = 12
        AD_GROUP_BID_MODIFIER = 13
        SHARED_SET = 14
        CAMPAIGN_SHARED_SET = 15
        ASSET = 16
        CUSTOMER_ASSET = 17
        CAMPAIGN_ASSET = 18
        AD_GROUP_ASSET = 19
        COMBINED_AUDIENCE = 20
        ASSET_GROUP = 21
        ASSET_SET = 22
        CAMPAIGN_ASSET_SET = 23


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AccountBudgetProposalStatusEnum",
    },
)


class AccountBudgetProposalStatusEnum(proto.Message):
    r"""Message describing AccountBudgetProposal statuses."""

    class AccountBudgetProposalStatus(proto.Enum):
        r"""The possible statuses of an AccountBudgetProposal.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PENDING (2):
                The proposal is pending approval.
            APPROVED_HELD (3):
                The proposal has been approved but the
                corresponding billing setup has not.  This can
                occur for proposals that set up the first budget
                when signing up for billing or when performing a
                change of bill-to operation.
            APPROVED (4):
                The proposal has been approved.
            CANCELLED (5):
                The proposal has been cancelled by the user.
            REJECTED (6):
                The proposal has been rejected by the user,
                for example, by rejecting an acceptance email.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PENDING = 2
        APPROVED_HELD = 3
        APPROVED = 4
        CANCELLED = 5
        REJECTED = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "ConversionActionTypeEnum",
    },
)


class ConversionActionTypeEnum(proto.Message):
    r"""Container for enum describing possible types of a conversion
    action.

    """

    class ConversionActionType(proto.Enum):
        r"""Possible types of a conversion action.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            AD_CALL (2):
                Conversions that occur when a user clicks on
                an ad's call extension.
            CLICK_TO_CALL (3):
                Conversions that occur when a user on a
                mobile device clicks a phone number.
            GOOGLE_PLAY_DOWNLOAD (4):
                Conversions that occur when a user downloads
                a mobile app from the Google Play Store.
            GOOGLE_PLAY_IN_APP_PURCHASE (5):
                Conversions that occur when a user makes a
                purchase in an app through Android billing.
            UPLOAD_CALLS (6):
                Call conversions that are tracked by the
                advertiser and uploaded.
            UPLOAD_CLICKS (7):
                Conversions that are tracked by the
                advertiser and uploaded with attributed clicks.
            WEBPAGE (8):
                Conversions that occur on a webpage.
            WEBSITE_CALL (9):
                Conversions that occur when a user calls a
                dynamically-generated phone number from an
                advertiser's website.
            STORE_SALES_DIRECT_UPLOAD (10):
                Store Sales conversion based on first-party
                or third-party merchant data uploads.
                Only customers on the allowlist can use store
                sales direct upload types.
            STORE_SALES (11):
                Store Sales conversion based on first-party
                or third-party merchant data uploads and/or from
                in-store purchases using cards from payment
                networks.
                Only customers on the allowlist can use store
                sales types. Read only.
            FIREBASE_ANDROID_FIRST_OPEN (12):
                Android app first open conversions tracked
                through Firebase.
            FIREBASE_ANDROID_IN_APP_PURCHASE (13):
                Android app in app purchase conversions
                tracked through Firebase.
            FIREBASE_ANDROID_CUSTOM (14):
                Android app custom conversions tracked
                through Firebase.
            FIREBASE_IOS_FIRST_OPEN (15):
                iOS app first open conversions tracked
                through Firebase.
            FIREBASE_IOS_IN_APP_PURCHASE (16):
                iOS app in app purchase conversions tracked
                through Firebase.
            FIREBASE_IOS_CUSTOM (17):
                iOS app custom conversions tracked through
                Firebase.
            THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN (18):
                Android app first open conversions tracked
                through Third Party App Analytics.
            THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE (19):
                Android app in app purchase conversions
                tracked through Third Party App Analytics.
            THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM (20):
                Android app custom conversions tracked
                through Third Party App Analytics.
            THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN (21):
                iOS app first open conversions tracked
                through Third Party App Analytics.
            THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE (22):
                iOS app in app purchase conversions tracked
                through Third Party App Analytics.
            THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM (23):
                iOS app custom conversions tracked through
                Third Party App Analytics.
            ANDROID_APP_PRE_REGISTRATION (24):
                Conversions that occur when a user
                pre-registers a mobile app from the Google Play
                Store. Read only.
            ANDROID_INSTALLS_ALL_OTHER_APPS (25):
                Conversions that track all Google Play
                downloads which aren't tracked by an
                app-specific type. Read only.
            FLOODLIGHT_ACTION (26):
                Floodlight activity that counts the number of
                times that users have visited a particular
                webpage after seeing or clicking on one of an
                advertiser's ads. Read only.
            FLOODLIGHT_TRANSACTION (27):
                Floodlight activity that tracks the number of
                sales made or the number of items purchased. Can
                also capture the total value of each sale. Read
                only.
            GOOGLE_HOSTED (28):
                Conversions that track local actions from
                Google's products and services after interacting
                with an ad. Read only.
            LEAD_FORM_SUBMIT (29):
                Conversions reported when a user submits a
                lead form. Read only.
            SALESFORCE (30):
                Deprecated: The Salesforce integration will
                be going away and replaced with an improved way
                to import your conversions from Salesforce.
                - see
                  https://support.google.com/google-ads/answer/14728349
            SEARCH_ADS_360 (31):
                Conversions imported from Search Ads 360
                Floodlight data. Read only.
            SMART_CAMPAIGN_AD_CLICKS_TO_CALL (32):
                Call conversions that occur on Smart campaign
                Ads without call tracking setup, using Smart
                campaign custom criteria. Read only.
            SMART_CAMPAIGN_MAP_CLICKS_TO_CALL (33):
                The user clicks on a call element within
                Google Maps. Smart campaign only. Read only.
            SMART_CAMPAIGN_MAP_DIRECTIONS (34):
                The user requests directions to a business
                location within Google Maps. Smart campaign
                only. Read only.
            SMART_CAMPAIGN_TRACKED_CALLS (35):
                Call conversions that occur on Smart campaign
                Ads with call tracking setup, using Smart
                campaign custom criteria. Read only.
            STORE_VISITS (36):
                Conversions that occur when a user visits an
                advertiser's retail store. Read only.
            WEBPAGE_CODELESS (37):
                Conversions created from website events (such
                as form submissions or page loads), that don't
                use individually coded event snippets. Read
                only.
            UNIVERSAL_ANALYTICS_GOAL (38):
                Conversions that come from linked Universal
                Analytics goals.
            UNIVERSAL_ANALYTICS_TRANSACTION (39):
                Conversions that come from linked Universal
                Analytics transactions.
            GOOGLE_ANALYTICS_4_CUSTOM (40):
                Conversions that come from linked Google
                Analytics 4 custom event conversions.
            GOOGLE_ANALYTICS_4_PURCHASE (41):
                Conversions that come from linked Google
                Analytics 4 purchase conversions.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_CALL = 2
        CLICK_TO_CALL = 3
        GOOGLE_PLAY_DOWNLOAD = 4
        GOOGLE_PLAY_IN_APP_PURCHASE = 5
        UPLOAD_CALLS = 6
        UPLOAD_CLICKS = 7
        WEBPAGE = 8
        WEBSITE_CALL = 9
        STORE_SALES_DIRECT_UPLOAD = 10
        STORE_SALES = 11
        FIREBASE_ANDROID_FIRST_OPEN = 12
        FIREBASE_ANDROID_IN_APP_PURCHASE = 13
        FIREBASE_ANDROID_CUSTOM = 14
        FIREBASE_IOS_FIRST_OPEN = 15
        FIREBASE_IOS_IN_APP_PURCHASE = 16
        FIREBASE_IOS_CUSTOM = 17
        THIRD_PARTY_APP_ANALYTICS_ANDROID_FIRST_OPEN = 18
        THIRD_PARTY_APP_ANALYTICS_ANDROID_IN_APP_PURCHASE = 19
        THIRD_PARTY_APP_ANALYTICS_ANDROID_CUSTOM = 20
        THIRD_PARTY_APP_ANALYTICS_IOS_FIRST_OPEN = 21
        THIRD_PARTY_APP_ANALYTICS_IOS_IN_APP_PURCHASE = 22
        THIRD_PARTY_APP_ANALYTICS_IOS_CUSTOM = 23
        ANDROID_APP_PRE_REGISTRATION = 24
        ANDROID_INSTALLS_ALL_OTHER_APPS = 25
        FLOODLIGHT_ACTION = 26
        FLOODLIGHT_TRANSACTION = 27
        GOOGLE_HOSTED = 28
        LEAD_FORM_SUBMIT = 29
        SALESFORCE = 30
        SEARCH_ADS_360 = 31
        SMART_CAMPAIGN_AD_CLICKS_TO_CALL = 32
        SMART_CAMPAIGN_MAP_CLICKS_TO_CALL = 33
        SMART_CAMPAIGN_MAP_DIRECTIONS = 34
        SMART_CAMPAIGN_TRACKED_CALLS = 35
        STORE_VISITS = 36
        WEBPAGE_CODELESS = 37
        UNIVERSAL_ANALYTICS_GOAL = 38
        UNIVERSAL_ANALYTICS_TRANSACTION = 39
        GOOGLE_ANALYTICS_4_CUSTOM = 40
        GOOGLE_ANALYTICS_4_PURCHASE = 41


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import ad_type_infos
from google.ads.googleads.v22.common.types import custom_parameter
from google.ads.googleads.v22.common.types import final_app_url
from google.ads.googleads.v22.common.types import url_collection
from google.ads.googleads.v22.enums.types import ad_type
from google.ads.googleads.v22.enums.types import device
from google.ads.googleads.v22.enums.types import system_managed_entity_source


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "Ad",
    },
)


class Ad(proto.Message):
    r"""An ad.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad. Ad resource names
            have the form:

            ``customers/{customer_id}/ads/{ad_id}``
        id (int):
            Output only. The ID of the ad.

            This field is a member of `oneof`_ ``_id``.
        final_urls (MutableSequence[str]):
            The list of possible final URLs after all
            cross-domain redirects for the ad.
        final_app_urls (MutableSequence[google.ads.googleads.v22.common.types.FinalAppUrl]):
            A list of final app URLs that will be used on
            mobile if the user has the specific app
            installed.
        final_mobile_urls (MutableSequence[str]):
            The list of possible final mobile URLs after
            all cross-domain redirects for the ad.
        tracking_url_template (str):
            The URL template for constructing a tracking
            URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        final_url_suffix (str):
            The suffix to use when constructing a final
            URL.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        url_custom_parameters (MutableSequence[google.ads.googleads.v22.common.types.CustomParameter]):
            The list of mappings that can be used to substitute custom
            parameter tags in a ``tracking_url_template``,
            ``final_urls``, or ``mobile_final_urls``. For mutates, use
            url custom parameter operations.
        display_url (str):
            The URL that appears in the ad description
            for some ad formats.

            This field is a member of `oneof`_ ``_display_url``.
        type_ (google.ads.googleads.v22.enums.types.AdTypeEnum.AdType):
            Output only. The type of ad.
        added_by_google_ads (bool):
            Output only. Indicates if this ad was
            automatically added by Google Ads and not by a
            user. For example, this could happen when ads
            are automatically created as suggestions for new
            ads based on knowledge of how existing ads are
            performing.

            This field is a member of `oneof`_ ``_added_by_google_ads``.
        device_preference (google.ads.googleads.v22.enums.types.DeviceEnum.Device):
            The device preference for the ad. You can
            only specify a preference for mobile devices.
            When this preference is set the ad will be
            preferred over other ads when being displayed on
            a mobile device. The ad can still be displayed
            on other device types, for example, if no other
            ads are available. If unspecified (no device
            preference), all devices are targeted. This is
            only supported by some ad types.
        url_collections (MutableSequence[google.ads.googleads.v22.common.types.UrlCollection]):
            Additional URLs for the ad that are tagged
            with a unique identifier that can be referenced
            from other fields in the ad.
        name (str):
            Immutable. The name of the ad. This is only
            used to be able to identify the ad. It does not
            need to be unique and does not affect the served
            ad. The name field is currently only supported
            for DisplayUploadAd, ImageAd,
            LegacyAppInstallAd, ShoppingComparisonListingAd,
            VideoAd, VideoResponsiveAd and DemandGen ads.

            This field is a member of `oneof`_ ``_name``.
        system_managed_resource_source (google.ads.googleads.v22.enums.types.SystemManagedResourceSourceEnum.SystemManagedResourceSource):
            Output only. If this ad is system managed,
            then this field will indicate the source. This
            field is read-only.
        text_ad (google.ads.googleads.v22.common.types.TextAdInfo):
            Immutable. Details pertaining to a text ad.

            This field is a member of `oneof`_ ``ad_data``.
        expanded_text_ad (google.ads.googleads.v22.common.types.ExpandedTextAdInfo):
            Details pertaining to an expanded text ad.

            This field is a member of `oneof`_ ``ad_data``.
        call_ad (google.ads.googleads.v22.common.types.CallAdInfo):
            Details pertaining to a call ad.

            This field is a member of `oneof`_ ``ad_data``.
        expanded_dynamic_search_ad (google.ads.googleads.v22.common.types.ExpandedDynamicSearchAdInfo):
            Immutable. Details pertaining to an Expanded Dynamic Search
            Ad. This type of ad has its headline, final URLs, and
            display URL auto-generated at serving time according to
            domain name specific information provided by
            ``dynamic_search_ads_setting`` linked at the campaign level.

            This field is a member of `oneof`_ ``ad_data``.
        hotel_ad (google.ads.googleads.v22.common.types.HotelAdInfo):
            Details pertaining to a hotel ad.

            This field is a member of `oneof`_ ``ad_data``.
        shopping_smart_ad (google.ads.googleads.v22.common.types.ShoppingSmartAdInfo):
            Details pertaining to a Smart Shopping ad.

            This field is a member of `oneof`_ ``ad_data``.
        shopping_product_ad (google.ads.googleads.v22.common.types.ShoppingProductAdInfo):
            Details pertaining to a Shopping product ad.

            This field is a member of `oneof`_ ``ad_data``.
        image_ad (google.ads.googleads.v22.common.types.ImageAdInfo):
            Immutable. Details pertaining to an Image ad.

            This field is a member of `oneof`_ ``ad_data``.
        video_ad (google.ads.googleads.v22.common.types.VideoAdInfo):
            Details pertaining to a Video ad.

            This field is a member of `oneof`_ ``ad_data``.
        video_responsive_ad (google.ads.googleads.v22.common.types.VideoResponsiveAdInfo):
            Details pertaining to a Video responsive ad.

            This field is a member of `oneof`_ ``ad_data``.
        responsive_search_ad (google.ads.googleads.v22.common.types.ResponsiveSearchAdInfo):
            Details pertaining to a responsive search ad.

            This field is a member of `oneof`_ ``ad_data``.
        legacy_responsive_display_ad (google.ads.googleads.v22.common.types.LegacyResponsiveDisplayAdInfo):
            Details pertaining to a legacy responsive
            display ad.

            This field is a member of `oneof`_ ``ad_data``.
        app_ad (google.ads.googleads.v22.common.types.AppAdInfo):
            Details pertaining to an app ad.

            This field is a member of `oneof`_ ``ad_data``.
        legacy_app_install_ad (google.ads.googleads.v22.common.types.LegacyAppInstallAdInfo):
            Immutable. Details pertaining to a legacy app
            install ad.

            This field is a member of `oneof`_ ``ad_data``.
        responsive_display_ad (google.ads.googleads.v22.common.types.ResponsiveDisplayAdInfo):
            Details pertaining to a responsive display
            ad.

            This field is a member of `oneof`_ ``ad_data``.
        local_ad (google.ads.googleads.v22.common.types.LocalAdInfo):
            Details pertaining to a local ad.

            This field is a member of `oneof`_ ``ad_data``.
        display_upload_ad (google.ads.googleads.v22.common.types.DisplayUploadAdInfo):
            Details pertaining to a display upload ad.

            This field is a member of `oneof`_ ``ad_data``.
        app_engagement_ad (google.ads.googleads.v22.common.types.AppEngagementAdInfo):
            Details pertaining to an app engagement ad.

            This field is a member of `oneof`_ ``ad_data``.
        shopping_comparison_listing_ad (google.ads.googleads.v22.common.types.ShoppingComparisonListingAdInfo):
            Details pertaining to a Shopping Comparison
            Listing ad.

            This field is a member of `oneof`_ ``ad_data``.
        smart_campaign_ad (google.ads.googleads.v22.common.types.SmartCampaignAdInfo):
            Details pertaining to a Smart campaign ad.

            This field is a member of `oneof`_ ``ad_data``.
        app_pre_registration_ad (google.ads.googleads.v22.common.types.AppPreRegistrationAdInfo):
            Details pertaining to an app pre-registration
            ad.

            This field is a member of `oneof`_ ``ad_data``.
        demand_gen_multi_asset_ad (google.ads.googleads.v22.common.types.DemandGenMultiAssetAdInfo):
            Details pertaining to a Demand Gen multi
            asset ad.

            This field is a member of `oneof`_ ``ad_data``.
        demand_gen_carousel_ad (google.ads.googleads.v22.common.types.DemandGenCarouselAdInfo):
            Details pertaining to a Demand Gen carousel
            ad.

            This field is a member of `oneof`_ ``ad_data``.
        demand_gen_video_responsive_ad (google.ads.googleads.v22.common.types.DemandGenVideoResponsiveAdInfo):
            Details pertaining to a Demand Gen video
            responsive ad.

            This field is a member of `oneof`_ ``ad_data``.
        demand_gen_product_ad (google.ads.googleads.v22.common.types.DemandGenProductAdInfo):
            Details pertaining to a Demand Gen product
            ad.

            This field is a member of `oneof`_ ``ad_data``.
        travel_ad (google.ads.googleads.v22.common.types.TravelAdInfo):
            Details pertaining to a travel ad.

            This field is a member of `oneof`_ ``ad_data``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=37,
    )
    id: int = proto.Field(
        proto.INT64,
        number=40,
        optional=True,
    )
    final_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=41,
    )
    final_app_urls: MutableSequence[final_app_url.FinalAppUrl] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=35,
            message=final_app_url.FinalAppUrl,
        )
    )
    final_mobile_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=42,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=43,
        optional=True,
    )
    final_url_suffix: str = proto.Field(
        proto.STRING,
        number=44,
        optional=True,
    )
    url_custom_parameters: MutableSequence[custom_parameter.CustomParameter] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=10,
            message=custom_parameter.CustomParameter,
        )
    )
    display_url: str = proto.Field(
        proto.STRING,
        number=45,
        optional=True,
    )
    type_: ad_type.AdTypeEnum.AdType = proto.Field(
        proto.ENUM,
        number=5,
        enum=ad_type.AdTypeEnum.AdType,
    )
    added_by_google_ads: bool = proto.Field(
        proto.BOOL,
        number=46,
        optional=True,
    )
    device_preference: device.DeviceEnum.Device = proto.Field(
        proto.ENUM,
        number=20,
        enum=device.DeviceEnum.Device,
    )
    url_collections: MutableSequence[url_collection.UrlCollection] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=26,
            message=url_collection.UrlCollection,
        )
    )
    name: str = proto.Field(
        proto.STRING,
        number=47,
        optional=True,
    )
    system_managed_resource_source: (
        system_managed_entity_source.SystemManagedResourceSourceEnum.SystemManagedResourceSource
    ) = proto.Field(
        proto.ENUM,
        number=27,
        enum=system_managed_entity_source.SystemManagedResourceSourceEnum.SystemManagedResourceSource,
    )
    text_ad: ad_type_infos.TextAdInfo = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="ad_data",
        message=ad_type_infos.TextAdInfo,
    )
    expanded_text_ad: ad_type_infos.ExpandedTextAdInfo = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="ad_data",
        message=ad_type_infos.ExpandedTextAdInfo,
    )
    call_ad: ad_type_infos.CallAdInfo = proto.Field(
        proto.MESSAGE,
        number=49,
        oneof="ad_data",
        message=ad_type_infos.CallAdInfo,
    )
    expanded_dynamic_search_ad: ad_type_infos.ExpandedDynamicSearchAdInfo = (
        proto.Field(
            proto.MESSAGE,
            number=14,
            oneof="ad_data",
            message=ad_type_infos.ExpandedDynamicSearchAdInfo,
        )
    )
    hotel_ad: ad_type_infos.HotelAdInfo = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="ad_data",
        message=ad_type_infos.HotelAdInfo,
    )
    shopping_smart_ad: ad_type_infos.ShoppingSmartAdInfo = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="ad_data",
        message=ad_type_infos.ShoppingSmartAdInfo,
    )
    shopping_product_ad: ad_type_infos.ShoppingProductAdInfo = proto.Field(
        proto.MESSAGE,
        number=18,
        oneof="ad_data",
        message=ad_type_infos.ShoppingProductAdInfo,
    )
    image_ad: ad_type_infos.ImageAdInfo = proto.Field(
        proto.MESSAGE,
        number=22,
        oneof="ad_data",
        message=ad_type_infos.ImageAdInfo,
    )
    video_ad: ad_type_infos.VideoAdInfo = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="ad_data",
        message=ad_type_infos.VideoAdInfo,
    )
    video_responsive_ad: ad_type_infos.VideoResponsiveAdInfo = proto.Field(
        proto.MESSAGE,
        number=39,
        oneof="ad_data",
        message=ad_type_infos.VideoResponsiveAdInfo,
    )
    responsive_search_ad: ad_type_infos.ResponsiveSearchAdInfo = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="ad_data",
        message=ad_type_infos.ResponsiveSearchAdInfo,
    )
    legacy_responsive_display_ad: (
        ad_type_infos.LegacyResponsiveDisplayAdInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=28,
        oneof="ad_data",
        message=ad_type_infos.LegacyResponsiveDisplayAdInfo,
    )
    app_ad: ad_type_infos.AppAdInfo = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="ad_data",
        message=ad_type_infos.AppAdInfo,
    )
    legacy_app_install_ad: ad_type_infos.LegacyAppInstallAdInfo = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="ad_data",
        message=ad_type_infos.LegacyAppInstallAdInfo,
    )
    responsive_display_ad: ad_type_infos.ResponsiveDisplayAdInfo = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="ad_data",
        message=ad_type_infos.ResponsiveDisplayAdInfo,
    )
    local_ad: ad_type_infos.LocalAdInfo = proto.Field(
        proto.MESSAGE,
        number=32,
        oneof="ad_data",
        message=ad_type_infos.LocalAdInfo,
    )
    display_upload_ad: ad_type_infos.DisplayUploadAdInfo = proto.Field(
        proto.MESSAGE,
        number=33,
        oneof="ad_data",
        message=ad_type_infos.DisplayUploadAdInfo,
    )
    app_engagement_ad: ad_type_infos.AppEngagementAdInfo = proto.Field(
        proto.MESSAGE,
        number=34,
        oneof="ad_data",
        message=ad_type_infos.AppEngagementAdInfo,
    )
    shopping_comparison_listing_ad: (
        ad_type_infos.ShoppingComparisonListingAdInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=36,
        oneof="ad_data",
        message=ad_type_infos.ShoppingComparisonListingAdInfo,
    )
    smart_campaign_ad: ad_type_infos.SmartCampaignAdInfo = proto.Field(
        proto.MESSAGE,
        number=48,
        oneof="ad_data",
        message=ad_type_infos.SmartCampaignAdInfo,
    )
    app_pre_registration_ad: ad_type_infos.AppPreRegistrationAdInfo = (
        proto.Field(
            proto.MESSAGE,
            number=50,
            oneof="ad_data",
            message=ad_type_infos.AppPreRegistrationAdInfo,
        )
    )
    demand_gen_multi_asset_ad: ad_type_infos.DemandGenMultiAssetAdInfo = (
        proto.Field(
            proto.MESSAGE,
            number=62,
            oneof="ad_data",
            message=ad_type_infos.DemandGenMultiAssetAdInfo,
        )
    )
    demand_gen_carousel_ad: ad_type_infos.DemandGenCarouselAdInfo = proto.Field(
        proto.MESSAGE,
        number=63,
        oneof="ad_data",
        message=ad_type_infos.DemandGenCarouselAdInfo,
    )
    demand_gen_video_responsive_ad: (
        ad_type_infos.DemandGenVideoResponsiveAdInfo
    ) = proto.Field(
        proto.MESSAGE,
        number=64,
        oneof="ad_data",
        message=ad_type_infos.DemandGenVideoResponsiveAdInfo,
    )
    demand_gen_product_ad: ad_type_infos.DemandGenProductAdInfo = proto.Field(
        proto.MESSAGE,
        number=61,
        oneof="ad_data",
        message=ad_type_infos.DemandGenProductAdInfo,
    )
    travel_ad: ad_type_infos.TravelAdInfo = proto.Field(
        proto.MESSAGE,
        number=54,
        oneof="ad_data",
        message=ad_type_infos.TravelAdInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "UserListErrorEnum",
    },
)


class UserListErrorEnum(proto.Message):
    r"""Container for enum describing possible user list errors."""

    class UserListError(proto.Enum):
        r"""Enum describing possible user list errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED (2):
                Creating and updating external remarketing
                user lists is not supported.
            CONCRETE_TYPE_REQUIRED (3):
                Concrete type of user list is required.
            CONVERSION_TYPE_ID_REQUIRED (4):
                Creating/updating user list conversion types
                requires specifying the conversion type Id.
            DUPLICATE_CONVERSION_TYPES (5):
                Remarketing user list cannot have duplicate
                conversion types.
            INVALID_CONVERSION_TYPE (6):
                Conversion type is invalid/unknown.
            INVALID_DESCRIPTION (7):
                User list description is empty or invalid.
            INVALID_NAME (8):
                User list name is empty or invalid.
            INVALID_TYPE (9):
                Type of the UserList does not match.
            CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND (10):
                Embedded logical user lists are not allowed.
            INVALID_USER_LIST_LOGICAL_RULE_OPERAND (11):
                User list rule operand is invalid.
            NAME_ALREADY_USED (12):
                Name is already being used for another user
                list for the account.
            NEW_CONVERSION_TYPE_NAME_REQUIRED (13):
                Name is required when creating a new
                conversion type.
            CONVERSION_TYPE_NAME_ALREADY_USED (14):
                The given conversion type name has been used.
            OWNERSHIP_REQUIRED_FOR_SET (15):
                Only an owner account may edit a user list.
            USER_LIST_MUTATE_NOT_SUPPORTED (16):
                Creating user list without setting type in oneof user_list
                field, or creating/updating read-only user list types is not
                allowed.
            INVALID_RULE (17):
                Rule is invalid.
            INVALID_DATE_RANGE (27):
                The specified date range is empty.
            CAN_NOT_MUTATE_SENSITIVE_USERLIST (28):
                A UserList which is privacy sensitive or
                legal rejected cannot be mutated by external
                users.
            MAX_NUM_RULEBASED_USERLISTS (29):
                Maximum number of rulebased user lists a
                customer can have.
            CANNOT_MODIFY_BILLABLE_RECORD_COUNT (30):
                BasicUserList's billable record field cannot
                be modified once it is set.
            APP_ID_NOT_SET (31):
                crm_based_user_list.app_id field must be set when
                upload_key_type is MOBILE_ADVERTISING_ID.
            USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST (32):
                Name of the user list is reserved for system
                generated lists and cannot be used.
            ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA (37):
                Advertiser needs to be on the allow-list to
                use remarketing lists created from advertiser
                uploaded data (for example, Customer Match
                lists).
            RULE_TYPE_IS_NOT_SUPPORTED (34):
                The provided rule_type is not supported for the user list.
            CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND (35):
                Similar user list cannot be used as a logical
                user list operand.
            CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS (36):
                Logical user list should not have a mix of
                CRM based user list and other types of lists in
                its rules.
            APP_ID_NOT_ALLOWED (39):
                crm_based_user_list.app_id field can only be set when
                upload_key_type is MOBILE_ADVERTISING_ID.
            CANNOT_MUTATE_SYSTEM_LIST (40):
                Google system generated user lists cannot be
                mutated.
            MOBILE_APP_IS_SENSITIVE (41):
                The mobile app associated with the
                remarketing list is sensitive.
            SEED_LIST_DOES_NOT_EXIST (42):
                One or more given seed lists do not exist.
            INVALID_SEED_LIST_ACCESS_REASON (43):
                One or more given seed lists are not
                accessible to the current user.
            INVALID_SEED_LIST_TYPE (44):
                One or more given seed lists have an
                unsupported type.
            INVALID_COUNTRY_CODES (45):
                One or more invalid country codes are added
                to Lookalike UserList.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED = 2
        CONCRETE_TYPE_REQUIRED = 3
        CONVERSION_TYPE_ID_REQUIRED = 4
        DUPLICATE_CONVERSION_TYPES = 5
        INVALID_CONVERSION_TYPE = 6
        INVALID_DESCRIPTION = 7
        INVALID_NAME = 8
        INVALID_TYPE = 9
        CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND = 10
        INVALID_USER_LIST_LOGICAL_RULE_OPERAND = 11
        NAME_ALREADY_USED = 12
        NEW_CONVERSION_TYPE_NAME_REQUIRED = 13
        CONVERSION_TYPE_NAME_ALREADY_USED = 14
        OWNERSHIP_REQUIRED_FOR_SET = 15
        USER_LIST_MUTATE_NOT_SUPPORTED = 16
        INVALID_RULE = 17
        INVALID_DATE_RANGE = 27
        CAN_NOT_MUTATE_SENSITIVE_USERLIST = 28
        MAX_NUM_RULEBASED_USERLISTS = 29
        CANNOT_MODIFY_BILLABLE_RECORD_COUNT = 30
        APP_ID_NOT_SET = 31
        USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST = 32
        ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA = 37
        RULE_TYPE_IS_NOT_SUPPORTED = 34
        CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND = 35
        CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS = 36
        APP_ID_NOT_ALLOWED = 39
        CANNOT_MUTATE_SYSTEM_LIST = 40
        MOBILE_APP_IS_SENSITIVE = 41
        SEED_LIST_DOES_NOT_EXIST = 42
        INVALID_SEED_LIST_ACCESS_REASON = 43
        INVALID_SEED_LIST_TYPE = 44
        INVALID_COUNTRY_CODES = 45


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import access_invitation_status
from google.ads.googleads.v20.enums.types import access_role as gage_access_role


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerUserAccessInvitation",
    },
)


class CustomerUserAccessInvitation(proto.Message):
    r"""Represent an invitation to a new user on this customer
    account.

    Attributes:
        resource_name (str):
            Immutable. Name of the resource. Resource names have the
            form:
            ``customers/{customer_id}/customerUserAccessInvitations/{invitation_id}``
        invitation_id (int):
            Output only. The ID of the invitation.
            This field is read-only.
        access_role (google.ads.googleads.v20.enums.types.AccessRoleEnum.AccessRole):
            Immutable. Access role of the user.
        email_address (str):
            Immutable. Email address the invitation was
            sent to. This can differ from the email address
            of the account that accepts the invite.
        creation_date_time (str):
            Output only. Time invitation was created.
            This field is read-only.
            The format is "YYYY-MM-DD HH:MM:SS".
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".
        invitation_status (google.ads.googleads.v20.enums.types.AccessInvitationStatusEnum.AccessInvitationStatus):
            Output only. Invitation status of the user.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    invitation_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    access_role: gage_access_role.AccessRoleEnum.AccessRole = proto.Field(
        proto.ENUM,
        number=3,
        enum=gage_access_role.AccessRoleEnum.AccessRole,
    )
    email_address: str = proto.Field(
        proto.STRING,
        number=4,
    )
    creation_date_time: str = proto.Field(
        proto.STRING,
        number=5,
    )
    invitation_status: (
        access_invitation_status.AccessInvitationStatusEnum.AccessInvitationStatus
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=access_invitation_status.AccessInvitationStatusEnum.AccessInvitationStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "UrlCollection",
    },
)


class UrlCollection(proto.Message):
    r"""Collection of urls that is tagged with a unique identifier.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        url_collection_id (str):
            Unique identifier for this UrlCollection
            instance.

            This field is a member of `oneof`_ ``_url_collection_id``.
        final_urls (MutableSequence[str]):
            A list of possible final URLs.
        final_mobile_urls (MutableSequence[str]):
            A list of possible final mobile URLs.
        tracking_url_template (str):
            URL template for constructing a tracking URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
    """

    url_collection_id: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    final_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )
    final_mobile_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AppCampaignBiddingStrategyGoalTypeEnum",
    },
)


class AppCampaignBiddingStrategyGoalTypeEnum(proto.Message):
    r"""Container for enum describing goal towards which the bidding
    strategy of an app campaign should optimize for.

    """

    class AppCampaignBiddingStrategyGoalType(proto.Enum):
        r"""Goal type of App campaign BiddingStrategy.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            OPTIMIZE_INSTALLS_TARGET_INSTALL_COST (2):
                Aim to maximize the number of app installs.
                The cpa bid is the target cost per install.
            OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST (3):
                Aim to maximize the long term number of
                selected in-app conversions from app installs.
                The cpa bid is the target cost per install.
            OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST (4):
                Aim to maximize the long term number of
                selected in-app conversions from app installs.
                The cpa bid is the target cost per in-app
                conversion. Note that the actual cpa may seem
                higher than the target cpa at first, since the
                long term conversions haven't happened yet.
            OPTIMIZE_RETURN_ON_ADVERTISING_SPEND (5):
                Aim to maximize all conversions' value, for
                example, install + selected in-app conversions
                while achieving or exceeding target return on
                advertising spend.
            OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME (6):
                Aim to maximize the pre-registration of the
                app.
            OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST (7):
                Aim to maximize installation of the app
                without target cost-per-install.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OPTIMIZE_INSTALLS_TARGET_INSTALL_COST = 2
        OPTIMIZE_IN_APP_CONVERSIONS_TARGET_INSTALL_COST = 3
        OPTIMIZE_IN_APP_CONVERSIONS_TARGET_CONVERSION_COST = 4
        OPTIMIZE_RETURN_ON_ADVERTISING_SPEND = 5
        OPTIMIZE_PRE_REGISTRATION_CONVERSION_VOLUME = 6
        OPTIMIZE_INSTALLS_WITHOUT_TARGET_INSTALL_COST = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import conversion_action_category
from google.ads.googleads.v19.enums.types import conversion_origin


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CustomerConversionGoal",
    },
)


class CustomerConversionGoal(proto.Message):
    r"""Biddability control for conversion actions with a matching
    category and origin.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer conversion
            goal. Customer conversion goal resource names have the form:

            ``customers/{customer_id}/customerConversionGoals/{category}~{origin}``
        category (google.ads.googleads.v19.enums.types.ConversionActionCategoryEnum.ConversionActionCategory):
            The conversion category of this customer
            conversion goal. Only conversion actions that
            have this category will be included in this
            goal.
        origin (google.ads.googleads.v19.enums.types.ConversionOriginEnum.ConversionOrigin):
            The conversion origin of this customer
            conversion goal. Only conversion actions that
            have this conversion origin will be included in
            this goal.
        biddable (bool):
            The biddability of the customer conversion
            goal.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    category: (
        conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory,
    )
    origin: conversion_origin.ConversionOriginEnum.ConversionOrigin = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=conversion_origin.ConversionOriginEnum.ConversionOrigin,
        )
    )
    biddable: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds a price asset and associates it with an account."""


import argparse
from typing import Optional
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.asset_service import AssetOperation
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.common.types.asset_types import PriceAsset
from google.ads.googleads.v22.common.types.asset_types import PriceOffering
from google.ads.googleads.v22.enums.types.price_extension_price_unit import (
    PriceExtensionPriceUnitEnum,
)
from google.ads.googleads.v22.services.types.customer_asset_service import (
    CustomerAssetOperation,
)
from google.ads.googleads.v22.resources.types.customer_asset import (
    CustomerAsset,
)


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    # Create a new price asset.
    price_asset_resource_name: str = create_price_asset(client, customer_id)

    # Add the new price asset to the account.
    add_asset_to_account(client, customer_id, price_asset_resource_name)


def create_price_asset(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a price asset and returns its resource name.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a PriceAsset resource name.
    """
    # Create an asset operation.
    asset_operation: AssetOperation = client.get_type("AssetOperation")
    # Create an asset.
    asset: Asset = asset_operation.create
    asset.name = f"Price Asset #{uuid4()}"
    asset.tracking_url_template = "http://tracker.example.com/?u={lpurl}"

    # Create the price asset.
    price_asset: PriceAsset = asset.price_asset
    price_asset.type_ = client.enums.PriceExtensionTypeEnum.SERVICES
    # Price qualifier is optional.
    price_asset.price_qualifier = (
        client.enums.PriceExtensionPriceQualifierEnum.FROM
    )
    price_asset.language_code = "en"
    price_asset.price_offerings.extend(
        [
            create_price_offering(
                client,
                "Scrubs",
                "Body Scrub, Salt Scrub",
                "http://www.example.com/scrubs",
                "http://m.example.com/scrubs",
                60000000,  # 60 USD
                "USD",
                client.enums.PriceExtensionPriceUnitEnum.PER_HOUR,
            ),
            create_price_offering(
                client,
                "Hair Cuts",
                "Once a month",
                "http://www.example.com/haircuts",
                "http://m.example.com/haircuts",
                75000000,  # 75 USD
                "USD",
                client.enums.PriceExtensionPriceUnitEnum.PER_MONTH,
            ),
            create_price_offering(
                client,
                "Skin Care Package",
                "Four times a month",
                "http://www.example.com/skincarepackage",
                None,
                250000000,  # 250 USD
                "USD",
                client.enums.PriceExtensionPriceUnitEnum.PER_MONTH,
            ),
        ]
    )

    # Issue a mutate request to create the price asset.
    asset_service = client.get_service("AssetService")
    response = asset_service.mutate_assets(
        customer_id=customer_id, operations=[asset_operation]
    )
    resource_name: str = response.results[0].resource_name

    print(f"Created a price asset with resource name '{resource_name}'.")

    return resource_name


def create_price_offering(
    client: GoogleAdsClient,
    header: str,
    description: str,
    final_url: str,
    final_mobile_url: Optional[str],
    price_in_micros: int,
    currency_code: str,
    unit: PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit,
) -> PriceOffering:
    """Creates a PriceOffering instance and returns it.

    Args:
        client: an initialized GoogleAdsClient instance.
        header: The header of the price offering.
        description: The description of the price offering.
        final_url: The final_url of the price offering.
        final_mobile_url: The final_mobile_url of the price offering.
        price_in_micros: The price of the price offering.
        currency_code: The currency_code of the price offering.
        unit: The price unit of the price offering.

    Returns:
        A PriceOffering instance.
    """
    price_offering: PriceOffering = client.get_type("PriceOffering")
    price_offering.header = header
    price_offering.description = description
    price_offering.final_url = final_url
    # Check if this exists, since we pass None for one of the PriceOfferings
    # in the _create_price_asset method and assigning None to this field
    # raises an error.
    if final_mobile_url:
        price_offering.final_mobile_url = final_mobile_url
    price_offering.price.amount_micros = price_in_micros
    price_offering.price.currency_code = currency_code
    price_offering.unit = unit
    return price_offering


def add_asset_to_account(
    client: GoogleAdsClient, customer_id: str, price_asset_resource_name: str
) -> None:
    """Adds a new Asset to the given user account.

    Adding the Asset to an account allows it to serve in all campaigns under
    that account.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        price_asset_resource_name: a resource name for an Asset containing
            a PriceAsset.
    """
    # Create a customer asset operation.
    customer_asset_operation: CustomerAssetOperation = client.get_type(
        "CustomerAssetOperation"
    )
    # Create a customer asset, set its type to PRICE and attach price asset.
    asset: CustomerAsset = customer_asset_operation.create
    asset.field_type = client.enums.AssetFieldTypeEnum.PRICE
    asset.asset = price_asset_resource_name

    # Issue a mutate request to create the customer asset.
    customer_asset_service = client.get_service("CustomerAssetService")
    response = customer_asset_service.mutate_customer_assets(
        customer_id=customer_id, operations=[customer_asset_operation]
    )
    resource_name: str = response.results[0].resource_name

    print(
        "Created customer asset with resource name "
        f"'{response.results[0].resource_name}'."
    )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Add price asset for the specified customer id."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ImageErrorEnum",
    },
)


class ImageErrorEnum(proto.Message):
    r"""Container for enum describing possible image errors."""

    class ImageError(proto.Enum):
        r"""Enum describing possible image errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_IMAGE (2):
                The image is not valid.
            STORAGE_ERROR (3):
                The image could not be stored.
            BAD_REQUEST (4):
                There was a problem with the request.
            UNEXPECTED_SIZE (5):
                The image is not of legal dimensions.
            ANIMATED_NOT_ALLOWED (6):
                Animated image are not permitted.
            ANIMATION_TOO_LONG (7):
                Animation is too long.
            SERVER_ERROR (8):
                There was an error on the server.
            CMYK_JPEG_NOT_ALLOWED (9):
                Image cannot be in CMYK color format.
            FLASH_NOT_ALLOWED (10):
                Flash images are not permitted.
            FLASH_WITHOUT_CLICKTAG (11):
                Flash images must support clickTag.
            FLASH_ERROR_AFTER_FIXING_CLICK_TAG (12):
                A flash error has occurred after fixing the
                click tag.
            ANIMATED_VISUAL_EFFECT (13):
                Unacceptable visual effects.
            FLASH_ERROR (14):
                There was a problem with the flash image.
            LAYOUT_PROBLEM (15):
                Incorrect image layout.
            PROBLEM_READING_IMAGE_FILE (16):
                There was a problem reading the image file.
            ERROR_STORING_IMAGE (17):
                There was an error storing the image.
            ASPECT_RATIO_NOT_ALLOWED (18):
                The aspect ratio of the image is not allowed.
            FLASH_HAS_NETWORK_OBJECTS (19):
                Flash cannot have network objects.
            FLASH_HAS_NETWORK_METHODS (20):
                Flash cannot have network methods.
            FLASH_HAS_URL (21):
                Flash cannot have a Url.
            FLASH_HAS_MOUSE_TRACKING (22):
                Flash cannot use mouse tracking.
            FLASH_HAS_RANDOM_NUM (23):
                Flash cannot have a random number.
            FLASH_SELF_TARGETS (24):
                Ad click target cannot be '_self'.
            FLASH_BAD_GETURL_TARGET (25):
                GetUrl method should only use '_blank'.
            FLASH_VERSION_NOT_SUPPORTED (26):
                Flash version is not supported.
            FLASH_WITHOUT_HARD_CODED_CLICK_URL (27):
                Flash movies need to have hard coded click
                URL or clickTAG
            INVALID_FLASH_FILE (28):
                Uploaded flash file is corrupted.
            FAILED_TO_FIX_CLICK_TAG_IN_FLASH (29):
                Uploaded flash file can be parsed, but the
                click tag can not be fixed properly.
            FLASH_ACCESSES_NETWORK_RESOURCES (30):
                Flash movie accesses network resources
            FLASH_EXTERNAL_JS_CALL (31):
                Flash movie attempts to call external
                javascript code
            FLASH_EXTERNAL_FS_CALL (32):
                Flash movie attempts to call flash system
                commands
            FILE_TOO_LARGE (33):
                Image file is too large.
            IMAGE_DATA_TOO_LARGE (34):
                Image data is too large.
            IMAGE_PROCESSING_ERROR (35):
                Error while processing the image.
            IMAGE_TOO_SMALL (36):
                Image is too small.
            INVALID_INPUT (37):
                Input was invalid.
            PROBLEM_READING_FILE (38):
                There was a problem reading the image file.
            IMAGE_CONSTRAINTS_VIOLATED (39):
                Image constraints are violated, but details like
                ASPECT_RATIO_NOT_ALLOWED can't be provided. This happens
                when asset spec contains more than one constraint and
                different criteria of different constraints are violated.
            FORMAT_NOT_ALLOWED (40):
                Image format is not allowed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_IMAGE = 2
        STORAGE_ERROR = 3
        BAD_REQUEST = 4
        UNEXPECTED_SIZE = 5
        ANIMATED_NOT_ALLOWED = 6
        ANIMATION_TOO_LONG = 7
        SERVER_ERROR = 8
        CMYK_JPEG_NOT_ALLOWED = 9
        FLASH_NOT_ALLOWED = 10
        FLASH_WITHOUT_CLICKTAG = 11
        FLASH_ERROR_AFTER_FIXING_CLICK_TAG = 12
        ANIMATED_VISUAL_EFFECT = 13
        FLASH_ERROR = 14
        LAYOUT_PROBLEM = 15
        PROBLEM_READING_IMAGE_FILE = 16
        ERROR_STORING_IMAGE = 17
        ASPECT_RATIO_NOT_ALLOWED = 18
        FLASH_HAS_NETWORK_OBJECTS = 19
        FLASH_HAS_NETWORK_METHODS = 20
        FLASH_HAS_URL = 21
        FLASH_HAS_MOUSE_TRACKING = 22
        FLASH_HAS_RANDOM_NUM = 23
        FLASH_SELF_TARGETS = 24
        FLASH_BAD_GETURL_TARGET = 25
        FLASH_VERSION_NOT_SUPPORTED = 26
        FLASH_WITHOUT_HARD_CODED_CLICK_URL = 27
        INVALID_FLASH_FILE = 28
        FAILED_TO_FIX_CLICK_TAG_IN_FLASH = 29
        FLASH_ACCESSES_NETWORK_RESOURCES = 30
        FLASH_EXTERNAL_JS_CALL = 31
        FLASH_EXTERNAL_FS_CALL = 32
        FILE_TOO_LARGE = 33
        IMAGE_DATA_TOO_LARGE = 34
        IMAGE_PROCESSING_ERROR = 35
        IMAGE_TOO_SMALL = 36
        INVALID_INPUT = 37
        PROBLEM_READING_FILE = 38
        IMAGE_CONSTRAINTS_VIOLATED = 39
        FORMAT_NOT_ALLOWED = 40


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ChangeClientTypeEnum",
    },
)


class ChangeClientTypeEnum(proto.Message):
    r"""Container for enum describing the sources that the change
    event resource was made through.

    """

    class ChangeClientType(proto.Enum):
        r"""The source that the change_event resource was made through.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents an
                unclassified client type unknown in this
                version.
            GOOGLE_ADS_WEB_CLIENT (2):
                Changes made through the "ads.google.com".
                For example, changes made through campaign
                management.
            GOOGLE_ADS_AUTOMATED_RULE (3):
                Changes made through Google Ads automated
                rules.
            GOOGLE_ADS_SCRIPTS (4):
                Changes made through Google Ads scripts.
            GOOGLE_ADS_BULK_UPLOAD (5):
                Changes made by Google Ads bulk upload.
            GOOGLE_ADS_API (6):
                Changes made by Google Ads API.
            GOOGLE_ADS_EDITOR (7):
                Changes made by Google Ads Editor. This value
                is a placeholder. The API does not return these
                changes.
            GOOGLE_ADS_MOBILE_APP (8):
                Changes made by Google Ads mobile app.
            GOOGLE_ADS_RECOMMENDATIONS (9):
                Changes made through Google Ads
                recommendations.
            SEARCH_ADS_360_SYNC (10):
                Changes made through Search Ads 360 Sync.
            SEARCH_ADS_360_POST (11):
                Changes made through Search Ads 360 Post.
            INTERNAL_TOOL (12):
                Changes made through internal tools.
                For example, when a user sets a URL template on
                an entity like a Campaign, it's automatically
                wrapped with the SA360 Clickserver URL.
            OTHER (13):
                Types of changes that are not categorized,
                for example, changes made by coupon redemption
                through Google Ads.
            GOOGLE_ADS_RECOMMENDATIONS_SUBSCRIPTION (14):
                Changes made by subscribing to Google Ads
                recommendations.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        GOOGLE_ADS_WEB_CLIENT = 2
        GOOGLE_ADS_AUTOMATED_RULE = 3
        GOOGLE_ADS_SCRIPTS = 4
        GOOGLE_ADS_BULK_UPLOAD = 5
        GOOGLE_ADS_API = 6
        GOOGLE_ADS_EDITOR = 7
        GOOGLE_ADS_MOBILE_APP = 8
        GOOGLE_ADS_RECOMMENDATIONS = 9
        SEARCH_ADS_360_SYNC = 10
        SEARCH_ADS_360_POST = 11
        INTERNAL_TOOL = 12
        OTHER = 13
        GOOGLE_ADS_RECOMMENDATIONS_SUBSCRIPTION = 14


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "SimulationTypeEnum",
    },
)


class SimulationTypeEnum(proto.Message):
    r"""Container for enum describing the field a simulation
    modifies.

    """

    class SimulationType(proto.Enum):
        r"""Enum describing the field a simulation modifies.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CPC_BID (2):
                The simulation is for a CPC bid.
            CPV_BID (3):
                The simulation is for a CPV bid.
            TARGET_CPA (4):
                The simulation is for a CPA target.
            BID_MODIFIER (5):
                The simulation is for a bid modifier.
            TARGET_ROAS (6):
                The simulation is for a ROAS target.
            PERCENT_CPC_BID (7):
                The simulation is for a percent CPC bid.
            TARGET_IMPRESSION_SHARE (8):
                The simulation is for an impression share
                target.
            BUDGET (9):
                The simulation is for a budget.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CPC_BID = 2
        CPV_BID = 3
        TARGET_CPA = 4
        BID_MODIFIER = 5
        TARGET_ROAS = 6
        PERCENT_CPC_BID = 7
        TARGET_IMPRESSION_SHARE = 8
        BUDGET = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignGoalConfigErrorEnum",
    },
)


class CampaignGoalConfigErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign goal config
    errors.

    """

    class CampaignGoalConfigError(proto.Enum):
        r"""Enum describing possible campaign goal config errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            GOAL_NOT_FOUND (3):
                Goal is either removed or does not exist for
                this account.
            CAMPAIGN_NOT_FOUND (4):
                Campaign is either removed or does not exist.
            HIGH_LIFETIME_VALUE_PRESENT_BUT_VALUE_ABSENT (9):
                If high lifetime value is present then value
                should be present.
            HIGH_LIFETIME_VALUE_LESS_THAN_OR_EQUAL_TO_VALUE (10):
                High lifetime value should be greater than
                value.
            CUSTOMER_LIFECYCLE_OPTIMIZATION_CAMPAIGN_TYPE_NOT_SUPPORTED (11):
                When using customer lifecycle optimization
                goal, campaign type should be supported.
            CUSTOMER_NOT_ALLOWLISTED_FOR_RETENTION_ONLY (12):
                Customer must be allowlisted to use retention
                only goal.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        GOAL_NOT_FOUND = 3
        CAMPAIGN_NOT_FOUND = 4
        HIGH_LIFETIME_VALUE_PRESENT_BUT_VALUE_ABSENT = 9
        HIGH_LIFETIME_VALUE_LESS_THAN_OR_EQUAL_TO_VALUE = 10
        CUSTOMER_LIFECYCLE_OPTIMIZATION_CAMPAIGN_TYPE_NOT_SUPPORTED = 11
        CUSTOMER_NOT_ALLOWLISTED_FOR_RETENTION_ONLY = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v21.resources.types import (
    custom_conversion_goal as gagr_custom_conversion_goal,
)
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "MutateCustomConversionGoalsRequest",
        "CustomConversionGoalOperation",
        "MutateCustomConversionGoalsResponse",
        "MutateCustomConversionGoalResult",
    },
)


class MutateCustomConversionGoalsRequest(proto.Message):
    r"""Request message for
    [CustomConversionGoalService.MutateCustomConversionGoals][google.ads.googleads.v21.services.CustomConversionGoalService.MutateCustomConversionGoals].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose custom
            conversion goals are being modified.
        operations (MutableSequence[google.ads.googleads.v21.services.types.CustomConversionGoalOperation]):
            Required. The list of operations to perform
            on individual custom conversion goal.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v21.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CustomConversionGoalOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CustomConversionGoalOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class CustomConversionGoalOperation(proto.Message):
    r"""A single operation (create, remove) on a custom conversion
    goal.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v21.resources.types.CustomConversionGoal):
            Create operation: No resource name is
            expected for the new custom conversion goal

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v21.resources.types.CustomConversionGoal):
            Update operation: The custom conversion goal
            is expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed custom
            conversion goal is expected, in this format:

            'customers/{customer_id}/customConversionGoals/{goal_id}'

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: gagr_custom_conversion_goal.CustomConversionGoal = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_custom_conversion_goal.CustomConversionGoal,
    )
    update: gagr_custom_conversion_goal.CustomConversionGoal = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_custom_conversion_goal.CustomConversionGoal,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateCustomConversionGoalsResponse(proto.Message):
    r"""Response message for a custom conversion goal mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v21.services.types.MutateCustomConversionGoalResult]):
            All results for the mutate.
    """

    results: MutableSequence["MutateCustomConversionGoalResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="MutateCustomConversionGoalResult",
        )
    )


class MutateCustomConversionGoalResult(proto.Message):
    r"""The result for the custom conversion goal mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        custom_conversion_goal (google.ads.googleads.v21.resources.types.CustomConversionGoal):
            The mutated CustomConversionGoal with only mutable fields
            after mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    custom_conversion_goal: gagr_custom_conversion_goal.CustomConversionGoal = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message=gagr_custom_conversion_goal.CustomConversionGoal,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a channel-level seasonality adjustment for Smart Bidding.

The adjustment changes Smart Bidding behavior by the expected change in
conversion rate for the given future time interval.

For more information on using seasonality adjustments, see:
https://developers.google.com/google-ads/api/docs/campaigns/bidding/seasonality-adjustments
"""


import argparse
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.bidding_seasonality_adjustment import (
    BiddingSeasonalityAdjustment,
)
from google.ads.googleads.v22.services.services.bidding_seasonality_adjustment_service import (
    BiddingSeasonalityAdjustmentServiceClient,
)
from google.ads.googleads.v22.services.types.bidding_seasonality_adjustment_service import (
    BiddingSeasonalityAdjustmentOperation,
    MutateBiddingSeasonalityAdjustmentsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    start_date_time: str,
    end_date_time: str,
    conversion_rate_modifier: float,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        start_date_time: a str of the start date for the exclusion period.
        end_date_time: a str of the end date for the exclusion period.
        conversion_rate_modifier: the modifier to apply to conversions during
            the given time period.
    """
    # [START add_bidding_seasonality_adjustment]
    bidding_seasonality_adjustment_service: (
        BiddingSeasonalityAdjustmentServiceClient
    ) = client.get_service("BiddingSeasonalityAdjustmentService")
    operation: BiddingSeasonalityAdjustmentOperation = client.get_type(
        "BiddingSeasonalityAdjustmentOperation"
    )
    bidding_seasonality_adjustment: BiddingSeasonalityAdjustment = (
        operation.create
    )
    # A unique name is required for every seasonality adjustment.
    bidding_seasonality_adjustment.name = f"Seasonality adjustment #{uuid4()}"
    # The CHANNEL scope applies the conversion_rate_modifier to all campaigns of
    # specific advertising channel types. In this example, the
    # conversion_rate_modifier will only apply to Search campaigns. Use the
    # CAMPAIGN scope to instead limit the scope to specific campaigns.
    bidding_seasonality_adjustment.scope = (
        client.enums.SeasonalityEventScopeEnum.CHANNEL
    )
    bidding_seasonality_adjustment.advertising_channel_types.append(
        client.enums.AdvertisingChannelTypeEnum.SEARCH
    )
    # If setting scope CAMPAIGN, add individual campaign resource name(s)
    # according to the commented out line below.
    #
    # bidding_seasonality_adjustment.campaigns.append(
    #     "INSERT_CAMPAIGN_RESOURCE_NAME_HERE"
    # )

    bidding_seasonality_adjustment.start_date_time = start_date_time
    bidding_seasonality_adjustment.end_date_time = end_date_time
    # The conversion_rate_modifier is the expected future conversion rate
    # change. When this field is unset or set to 1.0, no adjustment will be
    # applied to traffic. The allowed range is 0.1 to 10.0.
    bidding_seasonality_adjustment.conversion_rate_modifier = (
        conversion_rate_modifier
    )

    response: MutateBiddingSeasonalityAdjustmentsResponse = (
        bidding_seasonality_adjustment_service.mutate_bidding_seasonality_adjustments(
            customer_id=customer_id, operations=[operation]
        )
    )

    resource_name: str = response.results[0].resource_name

    print(f"Added seasonality adjustment with resource name: '{resource_name}'")
    # [END add_bidding_seasonality_adjustment]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds a seasonality adjustment for conversions in Smart "
        "Bidding for the given time interval."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-s",
        "--start_date_time",
        type=str,
        required=True,
        help="The start date for the adjustment period, must be in the format: "
        "'yyyy-MM-dd HH:mm:ss'.",
    )
    parser.add_argument(
        "-e",
        "--end_date_time",
        type=str,
        required=True,
        help="The end date for the adjustment period, must be in the format: "
        "'yyyy-MM-dd HH:mm:ss'.",
    )
    parser.add_argument(
        "-m",
        "--conversion_rate_modifier",
        type=float,
        required=True,
        help="The conversion rate modifier that will be applied during the "
        "adjustment period. This value must be in the range 0.1 to 10.0.",
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.start_date_time,
            args.end_date_time,
            args.conversion_rate_modifier,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import offline_user_data


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "UploadUserDataRequest",
        "UserDataOperation",
        "UploadUserDataResponse",
    },
)


class UploadUserDataRequest(proto.Message):
    r"""Request message for
    [UserDataService.UploadUserData][google.ads.googleads.v22.services.UserDataService.UploadUserData]


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which to
            update the user data.
        operations (MutableSequence[google.ads.googleads.v22.services.types.UserDataOperation]):
            Required. The list of operations to be done.
        customer_match_user_list_metadata (google.ads.googleads.v22.common.types.CustomerMatchUserListMetadata):
            Metadata for data updates to a Customer Match
            user list.

            This field is a member of `oneof`_ ``metadata``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["UserDataOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="UserDataOperation",
    )
    customer_match_user_list_metadata: (
        offline_user_data.CustomerMatchUserListMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="metadata",
        message=offline_user_data.CustomerMatchUserListMetadata,
    )


class UserDataOperation(proto.Message):
    r"""Operation to be made for the UploadUserDataRequest.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v22.common.types.UserData):
            The list of user data to be appended to the
            user list.

            This field is a member of `oneof`_ ``operation``.
        remove (google.ads.googleads.v22.common.types.UserData):
            The list of user data to be removed from the
            user list.

            This field is a member of `oneof`_ ``operation``.
    """

    create: offline_user_data.UserData = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=offline_user_data.UserData,
    )
    remove: offline_user_data.UserData = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=offline_user_data.UserData,
    )


class UploadUserDataResponse(proto.Message):
    r"""Response message for
    [UserDataService.UploadUserData][google.ads.googleads.v22.services.UserDataService.UploadUserData]
    Uploads made through this service will not be visible under the
    'Segment members' section for the Customer Match List in the Google
    Ads UI.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        upload_date_time (str):
            The date time at which the request was received by API,
            formatted as "yyyy-mm-dd hh:mm:ss+\|-hh:mm", for example,
            "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_upload_date_time``.
        received_operations_count (int):
            Number of upload data operations received by
            API.

            This field is a member of `oneof`_ ``_received_operations_count``.
    """

    upload_date_time: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    received_operations_count: int = proto.Field(
        proto.INT32,
        number=4,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "PromotionBarcodeTypeEnum",
    },
)


class PromotionBarcodeTypeEnum(proto.Message):
    r"""Container for enum describing a promotion barcode type."""

    class PromotionBarcodeType(proto.Enum):
        r"""A promotion barcode type.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            AZTEC (2):
                Aztec 2D barcode format.
                Max 350 characters and no links
            CODABAR (3):
                CODABAR 1D format.
                Max 12 characters and no links. Supported
                characters include 0123456789-$:/.+ and optional
                start and end guards from ABCD.
            CODE39 (4):
                Code 39 1D format. Max 8 characters and no links. Supported
                characters include 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-.
                \*$/+%.
            CODE128 (5):
                Code 128 1D format.
                Max 18 ASCII characters only and no links
            DATA_MATRIX (6):
                Data Matrix 2D barcode format.
                Max 525 ISO-8859-1 characters only and no links
            EAN8 (7):
                EAN-8 1D format.
                The barcode value should be 7 digits (the check
                digit will be computed automatically) or 8
                digits (if you are providing your own check
                digit).
            EAN13 (8):
                EAN-13 1D format.
                The barcode value should be 12 digits (the check
                digit will be computed automatically) or 13
                digits (if you are providing your own check
                digit).
            ITF (9):
                ITF (Interleaved Two of Five) 1D format.
                Must be 14 digits long
            PDF417 (10):
                PDF417 format.
                Max 140 characters and no links
            UPC_A (11):
                UPC-A 1D format.
                The barcode value should be 11 digits (the check
                digit will be computed automatically) or 12
                digits (if you are providing your own check
                digit).
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AZTEC = 2
        CODABAR = 3
        CODE39 = 4
        CODE128 = 5
        DATA_MATRIX = 6
        EAN8 = 7
        EAN13 = 8
        ITF = 9
        PDF417 = 10
        UPC_A = 11


__all__ = tuple(sorted(__protobuf__.manifest))
